package com.fxkc.care.common;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;

@Data
@NoArgsConstructor
public class AjaxResult<T> {
    private int code;
    private String errorMsg;
    private T data;

    public AjaxResult(Type type, T data, String errorMsg) {
        super();
        this.code = type.getCode();
        this.data = data;
        this.errorMsg = ObjectUtils.isEmpty(errorMsg) ? type.getDesc() : errorMsg;
    }

    public AjaxResult(Type type) {
        this.code = type.getCode();
        this.errorMsg = type.getDesc();
    }

    public AjaxResult(Type type, T data) {
        this.code = type.getCode();
        this.errorMsg = type.getDesc();
        this.data = data;
    }

    public AjaxResult(int code, String message, T data) {
        this.code = code;
        this.errorMsg = message;
        this.data = data;
    }


    public static <T> AjaxResult<T> success(T data) {
        return new AjaxResult(Type.SUCCESS, data, null);
    }

    public static <T> AjaxResult<T> ok() {
        return new AjaxResult(Type.SUCCESS, "操作成功！", null);
    }

    public static <T> AjaxResult<T> error(String errorMsg) {
        return new AjaxResult(Type.ERROR, null, errorMsg);
    }

    public static <T> AjaxResult<T> error(Type type, String errorMsg) {
        return new AjaxResult(type, null, errorMsg);
    }

    public static <T> AjaxResult<T> error401(String errorMsg) {
        return new AjaxResult(Type.UNAUTHORIZED, null, errorMsg);
    }

    public static <T> AjaxResult<T> error601(String errorMsg) {
        return new AjaxResult(Type.INVALID_TOKEN, null, errorMsg);
    }

    public static <T> AjaxResult<T> error602(String errorMsg) {
        return new AjaxResult(Type.OTHER_CLIENT_LOGIN, null, errorMsg);
    }

    public static <T> AjaxResult<T> error603(String errorMsg) {
        return new AjaxResult(Type.TOKEN_TIMEOUT, null, errorMsg);
    }

    public enum Type {
        SUCCESS(1, "success"),
        ERROR(0, "error"),
        UNAUTHORIZED(401, "unauthorized"),
        INVALID_TOKEN(601, "invalidToken"),//不合法的token
        OTHER_CLIENT_LOGIN(602, "otherClientLogin"),//异地登录被挤下线
        TOKEN_TIMEOUT(603, "tokenTimeout"),//token超时失效
        ;

        private int code;
        private String desc;

        Type(int code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        public int getCode() {
            return this.code;
        }

        public String getDesc() {
            return this.desc;
        }
    }
}
