package com.fxkc.care.config;

import com.fxkc.care.common.AjaxResult;
import com.fxkc.care.utils.RequestUtil;
import com.fxkc.care.vo.UserInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Aspect
@Component
@Slf4j
public class RepeatSubmitAspect {
    @Autowired
    private RedisTemplate redisTemplate;
    @Value("${repeat.submit.period}")
    private String repeatSubmitPeriod;

    @Around("@annotation(repeatSubmitValidation)")
    public Object doAround(ProceedingJoinPoint proceedingJoinPoint, RepeatSubmitValidation repeatSubmitValidation)
            throws Throwable {
        UserInfoVO userInfo = RequestUtil.getUserInfo();
        if (repeatSubmitValidation != null) {
            // 类名
            String className = proceedingJoinPoint.getTarget().getClass().getName();
            // 方法名
            String methodName = proceedingJoinPoint.getSignature().getName();
            // redis-key
            String fullName = userInfo.getLoginName() + "-" + className + "." + methodName;
            Long result = redisTemplate.opsForValue().increment(fullName, 1);
            if (result > 1) {
                log.warn("**************** {} Method Repeat Submit ****************", className + "." + methodName);
                return AjaxResult.error("请勿重复提交！");
            }
            redisTemplate.expire(fullName, Integer.valueOf(repeatSubmitPeriod), TimeUnit.MILLISECONDS);
        }
        return proceedingJoinPoint.proceed();
    }
}
