package com.fxkc.care.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fxkc.care.common.AjaxResult;
import com.fxkc.care.config.RepeatSubmitValidation;
import com.fxkc.care.domain.UserInfo;
import com.fxkc.care.domain.UserRole;
import com.fxkc.care.dto.UserInfoAddDTO;
import com.fxkc.care.dto.UserInfoEditDTO;
import com.fxkc.care.service.IUserInfoService;
import com.fxkc.care.utils.RequestUtil;
import com.fxkc.care.vo.UserInfoVO;
import io.jsonwebtoken.lang.Assert;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 用户表 前端控制器
 * </p>
 *
 * @author JYH
 * @since 2024-12-14
 */
@RestController
@RequestMapping("/userInfo")
public class UserInfoController {

    @Autowired
    private IUserInfoService userInfoService;

    @ApiOperation("获取当前用户信息详情")
    @GetMapping("/currentUserInfo")
    public AjaxResult getCurrentUserInfo() {
        UserInfoVO userInfoVO = RequestUtil.getUserInfo();
        return AjaxResult.success(userInfoVO);
    }

//    @PostMapping("/editPwd")
//    @ApiOperation("修改密码")
//    public AjaxResult<Boolean> editPassword(@RequestBody UserInfo userInfo) {
//        Assert.notNull(userInfo.getPassword(), "密码不得为空！");
//        return AjaxResult.success(userInfoService.editPassword(userInfo));
//    }

//    @PostMapping("/page")
//    public AjaxResult page(@RequestBody UserInfo userInfo) {
//        return userInfoService.getPage(userInfo);
//    }

    @PostMapping("/add")
    @ApiOperation("新增用户")
    @RepeatSubmitValidation
    public AjaxResult add(@RequestBody @Validated UserInfoAddDTO dto) {
        return userInfoService.add(dto);
    }

    @PostMapping("/edit")
    @ApiOperation("编辑用户")
    @RepeatSubmitValidation
    public AjaxResult edit(@RequestBody UserInfoEditDTO dto) {
        return userInfoService.edit(dto);
    }

    @DeleteMapping("/delete/{id}")
    public AjaxResult delete(@PathVariable("id") String id) {
        return userInfoService.delete(id);
    }
}
