package com.fxkc.care.utils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class MultiThreadDateUtil {
    /**
     * 存放不同的日期模板格式的sdf的Map
     */
    private static ThreadLocal<Map<String, SimpleDateFormat>> sdfMap =
            new ThreadLocal<Map<String, SimpleDateFormat>>() {
                protected Map<String, SimpleDateFormat> initialValue() {
                    return new HashMap<String, SimpleDateFormat>();
                }
            };

    /**
     * 返回一个SimpleDateFormat,每个线程只会new一次pattern对应的sdf
     */
    private static SimpleDateFormat getSdf(String pattern) {
        Map<String, SimpleDateFormat> t = sdfMap.get();
        SimpleDateFormat sdf = t.get(pattern);
        if (sdf == null) {
            sdf = new SimpleDateFormat(pattern);
            t.put(pattern, sdf);
        }
        return sdf;
    }

    public static String format(Date date, String pattern) {
        return getSdf(pattern).format(date);
    }

    public static Date parse(String dateStr, String pattern) throws Exception {
        return getSdf(pattern).parse(dateStr);
    }

    public static String getPeriodStrBy2Date(Date date1, Date date2) {
        long between = date1.getTime() - date2.getTime();
        long day = between / (24 * 60 * 60 * 1000);
        long hour = between / (60 * 60 * 1000) - day * 24;
        long min = between / (60 * 1000) - day * 24 * 60 - hour * 60;
        long s = between / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60;
        StringBuilder sb = new StringBuilder();
        sb.append(day != 0L ? day + "天" : "")
                .append(hour != 0L ? hour + "小时" : "")
                .append(min != 0L ? min + "分钟" : "")
                .append(s != 0L ? s + "秒" : "");
        return sb.toString();
    }

    public static void main(String[] args) throws Exception {
        String date1str = "2024-07-01 14:25:55";
        String date2str = "2024-07-01 16:15:60";
        Date date1 = parse(date1str, "yyyy-MM-dd HH:mm:ss");
        Date date2 = parse(date2str, "yyyy-MM-dd HH:mm:ss");
        String periodStrBy2Date = getPeriodStrBy2Date(date2, date1);
        System.out.println("periodStrBy2Date = " + periodStrBy2Date);
    }

}

